unit demoAPI;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, WRAPI, ExtCtrls, Gauges;

type
  TWRDemo = class(TForm)
    FreqValue: TEdit;
    FreqLabel: TLabel;
    BtnMode: TButton;
    BFOUpDown: TUpDown;
    BFOLabel: TLabel;
    BFOValue: TEdit;
    VolumeLabel: TLabel;
    VolUpDown: TUpDown;
    BtnAttenuation: TButton;
    BtnMute: TButton;
    HeadingLabel: TLabel;
    BtnPower: TButton;
    BtnQuit: TButton;
    SignalGauge: TGauge;
    SignalLabel: TLabel;
    SignalPollTimer: TTimer;
    ModeValue: TLabel;
    AttenValue: TLabel;
    MuteValue: TLabel;
    PowerValue: TLabel;
    VolValue: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure BtnModeClick(Sender: TObject);
    procedure BtnPowerClick(Sender: TObject);
    procedure BtnMuteClick(Sender: TObject);
    procedure BtnAttenuationClick(Sender: TObject);
    procedure VolUpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure BtnQuitClick(Sender: TObject);
    procedure SignalPollTimerTimer(Sender: TObject);
    procedure BFOValueChange(Sender: TObject);
    procedure FreqValueChange(Sender: TObject);
  private
    { Private declarations }
    hRadio : Integer;
    Modes : Integer;
    Procedure DisplayMode;
    Procedure DisplayAtten;
    Procedure DisplayMute;
    Procedure DisplayPower;
    Procedure DisplayVolume;
    Procedure DisplayBFO;
    Procedure SetFreqFromDisplay;
    Procedure CheckBFOValue;
  public
    { Public declarations }
  end;

var
  WRDemo: TWRDemo;

implementation

{$R *.DFM}

Procedure TWRDemo.DisplayVolume;
{
  Procedure Gets volume from WinRadio, displays it in the
  edit box, and sets the UPDown control's position as well.
}
Var
  iVolume : Integer;

Begin {DisplayVolume}
  iVolume := GetVolume(hRadio);
  VolValue.Caption := IntToStr(iVolume);
  VolUpDown.Position := iVolume
End; {DisplayVolume}

Procedure TWRDemo.DisplayMute;
{
  Procedure displays the mute setting
}
Begin {DisplayMute}
  If (GetMute(hRadio)) Then
    MuteValue.Caption := 'ON'
  Else
    MuteValue.Caption := 'OFF'
End;  {DisplayMute}

Procedure TWRDemo.DisplayAtten;
{
  Procedure displays the attenuation setting
}
Begin {DisplayAtten}
  If (GetAtten(hRadio)) Then
    AttenValue.Caption := 'Local'
  Else
    AttenValue.Caption := 'DX';
End;  {DisplayAtten}

Procedure TWRDemo.DisplayPower;
{
  Procedure displays the power setting
}
Begin {DisplayPower}
  If (GetPower(hRadio)) Then
    PowerValue.Caption := 'ON'
  Else
    PowerValue.Caption := 'OFF';
End;  {DisplayPower}

Procedure TWRDemo.DisplayBFO;
{
  Procedure Gets BFO value from WinRadio, displays it in the
  edit box, and sets the position of the UPDown control.
}
Var
  iBFO : Integer;

Begin {DisplayBFO}
  iBFO := GetBFOOffset(hRadio);
  BFOValue.Text := IntToStr(iBFO);
  BFOUpDown.Position := iBFO
End; {DisplayBFO}

Procedure TWRDemo.DisplayMode;
{
  Procedure updates the Mode displayed and controls the
  BFO up/down control enable.
}
Var
  iMode: Integer;
  ModeStr : Array[0..31] of Char;

Begin {DisplayMode}
  iMode := LoByte(GetMode(hRadio));
  
  {Set enabled status of BFO}
  BFOUpDown.Enabled := (iMode = 0);

  {Set enabled status of BFO value edit box}
  BFOValue.Enabled := BFOUpDown.Enabled;

  {Get the name of the current mode}
  GetModeString(hRadio, iMode, ModeStr, SizeOf(ModeStr));
  ModeValue.Caption := ModeStr
End;  {DisplayMode}

Procedure TWRDemo.SetFreqFromDisplay;
{
  Procedure updates the WinRadio frequency from the value
  in the edit box.
}
Var
  dwFreq : LongInt;
  Code : Integer;

Begin {SetFreqFromDisplay}
   Val(FreqValue.Text, dwFreq, Code);
   If (Code<>0) Then
     {Read the value back and re-display}
     FreqValue.Text := IntToStr(GetFrequency(hRadio))
   Else
     SetFrequency(hRadio,dwFreq)
End;  {SetFreqFromDisplay}

Procedure TWRDemo.CheckBFOValue;
{
  Procedure updates the WinRadio BFO Offset from the value
  in the edit box.
}
Var
  iBFO : Integer;
  Code : Integer;

Begin {CheckBFOValue}
   Val(BFOValue.Text, iBFO, Code);
   If (Code<>0) Then
     {Read the value back and re-display}
     BFOValue.Text := IntToStr(GetBFOOffset(hRadio))
   Else
     SetBFOOffset(hRadio, iBFO)
End;  {CheckBFOValue}

procedure TWRDemo.FormCreate(Sender: TObject);
Var
  FRadioInfo: TRadioInfo;

begin
  {Test if WinRadio hardware is present}

  {
    NOTE: If WinRadio software is running, OpenRadioDevice()
          reports no card(s) found, because WinRadio will
          already have established a handle. 
  }
  hRadio := OpenRadioDevice(0);
  If hRadio = 0 Then
    Begin
      MessageDlg('Could not get a handle to the WiNRADiO card.'+#13#10+
                 'If the WiNRADiO application is running, please'+#13#10+
                 'terminate it and then re-run this program.'+#13#10+#13#10+
                 'This program will now terminate.',
                  mtInformation, [mbOK], 0);
      Application.Terminate
    End;

  {Read the WinRadio card parameters into FRadioInfo}
  FRadioInfo.dwSize := SizeOf(TRadioInfo);
  GetRadioDeviceInfo(hRadio, @FRadioInfo);

  {Establish number of modes available}
  Modes := FRadioInfo.iNumModes;

  {Set the bounds on the BFO control}
  BFOUpDown.Min := -FRadioInfo.iMaxBFO;
  BFOUpDown.Max := FRadioInfo.iMaxBFO;
  DisplayBFO;

  {Set the bounds on the volume control}
  VolUpDown.Max := FRadioInfo.iMaxVolume;
  DisplayVolume;

  {Read current values from WinRadio and display them}
  FreqValue.Text := IntToStr(GetFrequency(hRadio));
  DisplayMode;
  DisplayAtten;
  DisplayMute;
  DisplayPower;
end;

procedure TWRDemo.BtnModeClick(Sender: TObject);
Var
  iMode : Integer;

begin
  iMode := LoByte(GetMode(hRadio));
  Inc(iMode);
  iMode := iMode MOD Modes;
  If (SetMode(hRadio, iMode)) Then
    DisplayMode
end;

procedure TWRDemo.BtnPowerClick(Sender: TObject);
Var
  fPower : Boolean;

begin
  fPower := GetPower(hRadio);
  fPower := Not(fPower);
  If (SetPower(hRadio, fPower)) Then
    DisplayPower
end;

procedure TWRDemo.BtnMuteClick(Sender: TObject);
Var
  fMute : Boolean;

begin
  fMute := GetMute(hRadio);
  fMute := Not(fMute);
  If (SetMute(hRadio, fMute)) Then
    DisplayMute
end;

procedure TWRDemo.BtnAttenuationClick(Sender: TObject);
Var
  fAtten : Boolean;

begin
  fAtten := GetAtten(hRadio);
  fAtten := Not(fAtten);
  If (SetAtten(hRadio, fAtten)) Then
    DisplayAtten
end;

procedure TWRDemo.VolUpDownClick(Sender: TObject; Button: TUDBtnType);
begin
  SetVolume(hRadio, VolUpDown.Position);
  DisplayVolume
end;

procedure TWRDemo.BtnQuitClick(Sender: TObject);
begin
  Close
end;

procedure TWRDemo.SignalPollTimerTimer(Sender: TObject);
begin
  SignalGauge.Progress := LoWord(GetSignalStrength(hRadio));
end;

procedure TWRDemo.BFOValueChange(Sender: TObject);
begin
  CheckBFOValue
end;

procedure TWRDemo.FreqValueChange(Sender: TObject);
begin
  SetFreqFromDisplay
end;

end.




