unit wrapi;

interface

uses WinTypes, WinProcs;

const
  { Features (capabilities) }

  RIF_USVERSION = $00000001;    { Set if hardware is US version }
  RIF_DSP       = $00000002;    { Set if there is a DSP on the card }
  RIF_LSBUSB    = $00000004;    { Set if has CW/LSB/USB instead of SSB }

  { WiNRADiO Hardware Versions }

  VER_STANDARD  = $100;         { old, obsolete constants }
  VER_STANDARD2 = $10A;
  VER_SPECMON   = $200;

  RHV_1000      = VER_STANDARD2;  { new preferred version constants }
  RHV_3000      = VER_SPECMON;

  { Modes }  { bit 1 set = FM mode! (mode and RMD_FMN <> 0) }

  RMD_SSB   = 0;        { WR-1000 }
  RMD_CW    = RMD_SSB;  { WR-3000, WR-1500 }
  RMD_AM    = 1;        { All }
  RMD_FMN   = 2;        { All }
  RMD_FMW   = 3;        { All }
  RMD_LSB   = 4;        { WR-3000, WR-1500 }
  RMD_USB   = 5;        { WR-3000, WR-1500 }

  { Mode Bandwidths }

  RBW_DEFAULT = 0;
  RBW_2500  = $0200;    { Default for LSB/USB (and CW for WR-3000) }
  RBW_6k    = $0300;    { Default for AM (and SSB for WR-1000) }
  RBW_17k   = $0500;    { Default for FMN }
  RBW_280k  = $0700;    { Default for FMW }

  { Hardware interfaces }

  RHI_ISA     = 0;

  { Audio sources }

  RAS_RADIO     = 0;
  RAS_DSP       = 1;

  { DSP input sources }

  RDS_RADIO     = 0;
  RDS_DATA      = 1;

  { Wave formats }

  RWF_8KHZ      = $00000001;
  RWF_11KHZ     = $00000002;
  RWF_16KHZ     = $00000004;

  RWF_8BIT      = $00010000;
  RWF_16BIT     = $00020000;

  RWF_MONO      = $01000000;

  { DSP Notification messages }

  RM_DSPREQ       = $7F00;
  RM_DMAACKWRITE  = $7F01;
  RM_DMAACKREAD   = $7F02;
  RM_DMAREQWRITE  = $7F03;
  RM_DMAREQREAD   = $7F04;
  RM_DMAPAUSED    = $7F05;
  RM_DMARESUMED   = $7F06;

type
  PModeList = ^TModeList;
  TModeList = array [0..0] of Integer;

  PRadioInfo = ^TRadioInfo;
  TRadioInfo = packed record
    dwSize: Longint;            { size of structure }
    dwFeatures: Longint;        { bit flags for extra features }
    wAPIVer: Word;              { driver version }
    wHWVer: Word;               { hardware version }
    dwMinFreq: Longint;         { minimum frequency receiver can tune to }
    dwMaxFreq: Longint;         { maximum frequency receiver can tune to }
    iFreqRes: Integer;          { resolution of receiver in Hz }
    iNumModes: Integer;         { number of modes that can be set }
    iMaxVolume: Integer;        { maximum volume level }
    iMaxBFO: Integer;           { maximum BFO offset range (+/- in Hz) }
    iMaxFMScanRate: Integer;    { maximum scan rate for FM scanning/sweeping }
    iMaxAMScanRate: Integer;    { maximum scan rate for AM scanning/sweeping }
    iHWInterface: Integer;      { hardware interface used (or IO address) }
    iDeviceNum: Integer;        { device number }
    iNumSources: Integer;       { number of selectable audio sources }
    iMaxIFShift: Integer;       { maximum IF shift range }
    dwWaveFormats: Longint;     { DSP Wave formats suppoted }
    iDSPSources: Integer;       { Number of selectable DSP input sources }
    lpSupportedModes: PModeList;{ array specifying all supported modes }
  end;

  PDSPHdr = ^TDSPHdr;
  TDSPHdr = record
    lpData: PChar;
    dwBufferLength: Longint;
    dwBytesTransferred: Longint;
    dwUser: Longint;
    fDone: LongBool;
  end;

  TRMDSPReq = record
    Msg: Cardinal;
    hRadio: Cardinal;
    dwReqCode: Longint;
    Result: Longint;
  end;

  TRMDMAAckRead = record
    Msg: Cardinal;
    hRadio: Cardinal;
    lpDSPHdr: PDSPHdr;
    Result: Longint;
  end;

  TRMDMAAckWrite = TRMDMAAckRead;

  TRMDMAReqRead = record
    Msg: Cardinal;
    hRadio: Cardinal;
    dwLength: Longint;
    Result: Longint;
  end;

  TRMDMAReqWrite = TRMDMAReqRead;

  TRMRadioAdded = record
    Msg: Cardinal;
    wDevNum: Cardinal;
    dwInterface: Longint;
    Result: Longint;
  end;

var
  OpenRadioDevice: function (iDeviceNum: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  CloseRadioDevice: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetRadioDeviceInfo: function (hRadio: Integer; lpInfo: PRadioInfo): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetSignalStrength: function (hRadio: Integer): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}

  SetFrequency: function (hRadio: Integer; dwFreq: Longint): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetMode: function (hRadio, iMode: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetVolume: function (hRadio, iVolume: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetAtten: function (hRadio: Integer; fAtten: Bool): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetMute: function (hRadio: Integer; fMute: Bool): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetPower: function (hRadio: Integer; fPower: Bool): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetScanMode: function (hRadio: Integer; fScan: Bool): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetBFOOffset: function (hRadio, iBFO: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetIFShift: function (hRadio, iIFShift: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetAudioSource: function (hRadio, iSource: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  SetDSPSource: function (hRadio, iSource: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}

  GetFrequency: function (hRadio: Integer): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetMode: function (hRadio: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetVolume: function (hRadio: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetAtten: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetMute: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetPower: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetScanMode: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetBFOOffset: function (hRadio: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetIFShift: function (hRadio: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetAudioSource: function (hRadio: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  GetDSPSource: function (hRadio: Integer): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}

  { API functions for DSP }

  dspOpen: function (hRadio: Integer; lpBootCode: Pointer; hWindow: HWND): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspClose: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspSendByte: function (hRadio: Integer; bData: Byte): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspReadByte: function (hRadio: Integer; var lpbData: Byte): Bool;
                    {$IFDEF WIN32} stdcall; {$ENDIF}
  dspOutWrite: function (hRadio: Integer; lpDSPHeader: PDSPHdr; cbDSPHeader: Integer): Bool;
                    {$IFDEF WIN32} stdcall; {$ENDIF}
  dspInAddBuffer: function (hRadio: Integer; lpDSPHeader: PDSPHdr; cbDSPHeader: Integer): Bool;
                    {$IFDEF WIN32} stdcall; {$ENDIF}
  dspPause: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspResume: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspOutReset: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspInReset: function (hRadio: Integer): Bool; {$IFDEF WIN32} stdcall; {$ENDIF}

  dspInOpen: function (hRadio: Integer; dwWaveFormat: Longint; hWindow: HWND): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}
  dspOutOpen: function (hRadio: Integer; dwWaveFormat: Longint; hWindow: HWND): Integer; {$IFDEF WIN32} stdcall; {$ENDIF}

  { miscellaneous API functions }

  GetModeString: function (hRadio, iMode: Integer; lpString: PChar; iMaxLength: Integer): PChar;
                    {$IFDEF WIN32} stdcall; {$ENDIF}
  {$IFDEF WIN32}
  GetModeStringA: function (hRadio, iMode: Integer; lpString: PAnsiChar; iMaxLength: Integer): PAnsiChar; stdcall;
  GetModeStringW: function (hRadio, iMode: Integer; lpString: PWideChar; iMaxLength: Integer): PWideChar; stdcall;
  {$ENDIF}

  OpenRadioCount: function: Integer; {$IFDEF WIN32} stdcall; {$ENDIF}

function WRAPILoaded: Boolean;

implementation

function Null0: Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
begin
  Null0 := 0;
end;

function Null1(Param1: Integer): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
begin
  Null1 := 0;
end;

function Null2a(Param1, Param2: Integer): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
begin
  Null2a := 0;
end;

function Null2b(Param1: Integer; Param2: Longint): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
begin
  Null2b := 0;
end;

function Null3(Param1: Integer; Param2: Longint; Param3: Integer): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
begin
  Null3 := 0;
end;

function Null4(Param1, Param2: Integer; Param3: Longint; Param4: Integer): Longint; {$IFDEF WIN32} stdcall; {$ENDIF}
begin
  Null4 := 0;
end;

var
  hWRAPI: THandle;
  OldExitProc: Pointer;

function GetWRAPIProcAddress(ProcName: PChar; DefAddr: TFarProc): TFarProc;
var
  pa: TFarProc;
begin
  GetWRAPIProcAddress := DefAddr;
  if (hWRAPI > 32) then
  begin
    pa := GetProcAddress(hWRAPI, ProcName);
    if (pa <> nil) then
      GetWRAPIProcAddress := pa;
  end;
end;

function WRAPILoaded: Boolean;
begin
  WRAPILoaded := hWRAPI >= 32;
end;

procedure APIExitProc; {$IFNDEF WIN32} far; {$ENDIF}
begin
  ExitProc := OldExitProc;
  FreeLibrary(hWRAPI);
end;

procedure APIInit;
var
  OldSEM: Integer;
begin
  OldSEM := SetErrorMode(SEM_NOOPENFILEERRORBOX);
  {$IFDEF WIN32}
  hWRAPI := LoadLibrary('WRAPI32.DLL');
  {$ELSE}
  hWRAPI := LoadLibrary('WRAPI.DLL');
  {$ENDIF}
  SetErrorMode(OldSEM);

  if (hWRAPI >= 32) then
  begin
    OldExitProc := ExitProc;
    ExitProc := @APIExitProc;
  end;

  @OpenRadioDevice := GetWRAPIProcAddress('OpenRadioDevice', @Null1);
  @CloseRadioDevice := GetWRAPIProcAddress('CloseRadioDevice', @Null1);
  @GetRadioDeviceInfo := GetWRAPIProcAddress('GetRadioDeviceInfo', @Null2b);
  @GetSignalStrength := GetWRAPIProcAddress('GetSignalStrength', @Null1);

  @SetFrequency := GetWRAPIProcAddress('SetFrequency', @Null2b);
  @SetMode := GetWRAPIProcAddress('SetMode', @Null2a);
  @SetVolume := GetWRAPIProcAddress('SetVolume', @Null2a);
  @SetAtten := GetWRAPIProcAddress('SetAtten', @Null2a);
  @SetMute := GetWRAPIProcAddress('SetMute', @Null2a);
  @SetPower := GetWRAPIProcAddress('SetPower', @Null2a);
  @SetScanMode := GetWRAPIProcAddress('SetScanMode', @Null2a);
  @SetBFOOffset := GetWRAPIProcAddress('SetBFOOffset', @Null2a);
  @SetIFShift := GetWRAPIProcAddress('SetIFShift', @Null2a);
  @SetAudioSource := GetWRAPIProcAddress('SetAudioSource', @Null2a);
  @SetDSPSource := GetWRAPIProcAddress('SetDSPSource', @Null2a);

  @GetFrequency := GetWRAPIProcAddress('GetFrequency', @Null1);
  @GetMode := GetWRAPIProcAddress('GetMode', @Null1);
  @GetVolume := GetWRAPIProcAddress('GetVolume', @Null1);
  @GetAtten := GetWRAPIProcAddress('GetAtten', @Null1);
  @GetMute := GetWRAPIProcAddress('GetMute', @Null1);
  @GetPower := GetWRAPIProcAddress('GetPower', @Null1);
  @GetScanMode := GetWRAPIProcAddress('GetScanMode', @Null1);
  @GetBFOOffset := GetWRAPIProcAddress('GetBFOOffset', @Null1);
  @GetIFShift := GetWRAPIProcAddress('GetIFShift', @Null1);
  @GetAudioSource := GetWRAPIProcAddress('GetAudioSource', @Null1);
  @GetDSPSource := GetWRAPIProcAddress('GetDSPSource', @Null1);

  @dspOpen := GetWRAPIProcAddress('dspOpen', @Null3);
  @dspClose := GetWRAPIProcAddress('dspClose', @Null1);
  @dspSendByte := GetWRAPIProcAddress('dspSendByte', @Null2a);
  @dspReadByte := GetWRAPIProcAddress('dspReadByte', @Null2b);
  @dspOutWrite := GetWRAPIProcAddress('dspOutWrite', @Null3);
  @dspInAddBuffer := GetWRAPIProcAddress('dspInAddBuffer', @Null3);
  @dspPause := GetWRAPIProcAddress('dspPause', @Null1);
  @dspResume := GetWRAPIProcAddress('dspResume', @Null1);
  @dspOutReset := GetWRAPIProcAddress('dspOutReset', @Null1);
  @dspInReset := GetWRAPIProcAddress('dspInReset', @Null1);

  @dspInOpen := GetWRAPIProcAddress('dspInOpen', @Null3);
  @dspOutOpen := GetWRAPIProcAddress('dspOutOpen', @Null3);

  {$IFDEF WIN32}
  @GetModeStringA := GetWRAPIProcAddress('GetModeStringA', @Null4);
  @GetModeStringW := GetWRAPIProcAddress('GetModeStringW', @Null4);
  @GetModeString := @GetModeStringA;
  {$ELSE}
  @GetModeString := GetWRAPIProcAddress('GetModeString', @Null4);
  {$ENDIF}

  @OpenRadioCount := GetWRAPIProcAddress('OpenRadioCount', @Null0);
end;

begin
  APIInit;
end.
